# {{ name }} (see github.com/python-medikit)

from medikit import require

python = require('python')
{%- for feature in features | sort %}
{{ feature }} = require('{{ feature }}')
{%- endfor %}

python.setup(
    name = '{{ name }}',
    description = '{{ description }}',
    license = '{{ license }}',
    url = '{{ url }}',
    download_url = '{{ download_url }}',
    author = '{{ author }}',
    author_email = '{{ author_email }}',
)

python.add_requirements(
  # Insert your requirements here. We suggest you use minor version requirements here, and freeze an exact version in
  # requirements*.txt (which, when does not exist, can be generated by medikit).
  {% for requirement in requirements %}
    '{{ requirement }}',
  {% endfor %}
)

# vim: ft=python: