language: python
python:
    - 3.5
    - 3.5-dev
    - 3.6
    - 3.6-dev
    - nightly
install:
    - make install-dev
    - pip install coveralls
script:
    - make clean test
after_success:
    - coveralls