from medikit.commands.init import InitCommand
from medikit.commands.pipeline import PipelineCommand
from medikit.commands.update import UpdateCommand

# XXX deprecated aliases
handle_init = InitCommand.handle
handle_pipeline = PipelineCommand.handle
handle_update = UpdateCommand.handle
