from medikit._version import __version__


def listen(*args, **kwargs):
    raise NotImplementedError('This is a stub.')


def require(*args):
    raise NotImplementedError('This is a stub.')


def pipeline(*args):
    raise NotImplementedError('This is a stub.')


__all__ = [
    '__version__',
]
